@@ -1,6 +1,6 @@
 # commandinterface.py
 # this module supplies the command line interface for plotting
- 
+
 #    Copyright (C) 2004 Jeremy S. Sanders
 #    Email: Jeremy Sanders <jeremy@jeremysanders.net>
 #
@@ -34,10 +34,9 @@ from .. import setting
 from .. import embed
 from .. import plugins
 from .. import utils
+from .. import datasets
 
-from . import datasets
 from . import operations
-from . import dataset_histo
 from . import mime
 from . import export
 
@@ -45,15 +44,16 @@ def _(text, disambiguation=None, context='CommandInterface'):
     """Translate text."""
     return qt4.QCoreApplication.translate(context, text, disambiguation)
 
-def registerImportCommand(name, method):
+def registerImportCommand(name, method, filenamearg=0):
     """Add command to command interface."""
     setattr(CommandInterface, name, method)
-    CommandInterface.safe_commands.append(name)
+    CommandInterface.import_commands.append(name)
+    CommandInterface.import_filenamearg[name] = filenamearg
 
 class CommandInterface(qt4.QObject):
     """Class provides command interface."""
 
-    # commands which are safe in any script
+    # commands which are safe in any script (excluding import commands)
     safe_commands = [
         'Action',
         'Add',
@@ -62,8 +62,10 @@ class CommandInterface(qt4.QObject):
         'CloneWidget',
         'CreateHistogram',
         'DatasetPlugin',
+        'FilterDatasets',
         'Get',
         'GetChildren',
+        'GetColormap',
         'GetData',
         'GetDataType',
         'GetDatasets',
@@ -77,7 +79,6 @@ class CommandInterface(qt4.QObject):
         'Rename',
         'ResolveReference',
         'Set',
-        'SetToReference',
         'SetData',
         'SetData2D',
         'SetData2DExpression',
@@ -85,21 +86,28 @@ class CommandInterface(qt4.QObject):
         'SetData2DXYFunc',
         'SetDataDateTime',
         'SetDataExpression',
+        'SetDataND',
         'SetDataRange',
         'SetDataText',
-        'SettingType',
+        'SetToReference',
         'SetVerbose',
+        'SettingType',
         'TagDatasets',
         'To',
         'WidgetType',
         ]
 
+    # commands for importing data
+    import_commands = []
+    # number of argument which contains filename
+    import_filenamearg = {}
+
     # commands which can modify disk, etc
-    unsafe_commands = (
-        'Export',        
+    unsafe_commands = [
+        'Export',
         'Print',
         'Save',
-        )
+        ]
 
     def __init__(self, document):
         """Initialise the interface."""
@@ -114,6 +122,7 @@ class CommandInterface(qt4.QObject):
 
         self.Root = embed.WidgetNode(self, 'widget', '/')
 
+    @qt4.pyqtSlot()
     def slotWipedDoc(self):
         """When the document is wiped, we change to the root widget."""
         self.To('/')
@@ -148,7 +157,8 @@ class CommandInterface(qt4.QObject):
 
         at = self.currentwidget
         if 'widget' in args_opt:
-            at = self.document.resolve(self.currentwidget, args_opt['widget'])
+            at = self.document.resolveWidgetPath(
+                self.currentwidget, args_opt['widget'])
             del args_opt['widget']
 
         op = operations.OperationWidgetAdd(at, widgettype, **args_opt)
@@ -165,15 +175,18 @@ class CommandInterface(qt4.QObject):
 	constants), a function of 1 or more variables, or a function
 	imported from an external python module.
 
-        ctype is "constant", "function", "import" or "colormap".
+        ctype is "constant", "function", "definition" (either
+        constant or function), "import", "color" or "colormap".
 
-        name is name of constant or colormap, "function(x, y, ...)"
-        or module name.
+        name is name of constant, color or colormap, "function(x, y,
+        ...)"  or module name.
 
         val is definition for constant or function (both are
         _strings_), or is a list of symbols for a module (comma
         separated items in a string). For a colormap, val is a list of
-        4-item tuples containing R,G,B,alpha values from 0 to 255.
+        4-item tuples containing R,G,B,alpha values from 0 to 255. For
+        a color this is a string with the format '#RRGGBB' or
+        '#RRGGBBAA'.
 
         if mode is 'appendalways', the custom value is appended to the
         end of the list even if there is one with the same name. If
@@ -181,26 +194,31 @@ class CommandInterface(qt4.QObject):
         same place in the list or is appended otherwise. If mode is
         'append', then an existing definition is deleted, and the new
         one appended to the end.
+
         """
 
         if ctype == 'colormap':
-            self.document.validateProcessColormap(val)
+            self.document.evaluate.validateProcessColormap(val)
         else:
             if not isinstance(val, cbasestr):
                 raise RuntimeError('Value should be string')
 
         if mode not in ('appendalways', 'append', 'replace'):
             raise RuntimeError('Invalid mode')
-        if ctype not in ('constant', 'import', 'function', 'colormap'):
+
+        try:
+            attr = operations.OperationSetCustom.type_to_attr[ctype]
+        except KeyError:
             raise RuntimeError('Invalid type')
 
-        vals = list( self.document.customs )
-        item = [ctype, name, val]
+        vals = list(getattr(self.document.evaluate, attr))
+
+        item = [name.strip(), val]
         if mode == 'appendalways':
             vals.append(item)
         else:
             # find any existing item
-            for i, (t, n, v) in enumerate(vals):
+            for i, (n, v) in enumerate(vals):
                 if n == name:
                     if mode == 'append':
                         del vals[i]
@@ -212,7 +230,7 @@ class CommandInterface(qt4.QObject):
                 # no existing item, so append
                 vals.append(item)
 
-        op = operations.OperationSetCustom(vals)
+        op = operations.OperationSetCustom(ctype, vals)
         self.document.applyOperation(op)
 
     def AddImportPath(self, directory):
@@ -223,14 +241,14 @@ class CommandInterface(qt4.QObject):
     def CloneWidget(self, widget, newparent, newname=None):
         """Clone the widget given, placing the copy in newparent and
         the name given.
-        
+
         newname is an optional new name to give it
 
         Returns new widget path
         """
 
-        widget = self.document.resolve(self.currentwidget, widget)
-        newparent = self.document.resolve(self.currentwidget, newparent)
+        widget = self.document.resolveWidgetPath(self.currentwidget, widget)
+        newparent = self.document.resolveWidgetPath(self.currentwidget, newparent)
         op = mime.OperationWidgetClone(widget, newparent, newname)
         w = self.document.applyOperation(op)
         return w.path
@@ -250,7 +268,7 @@ class CommandInterface(qt4.QObject):
           'none', 'smalltolarge' or 'largetosmall'
         errors is to calculate Poisson error bars
         """
-        op = dataset_histo.OperationDatasetHistogram(
+        op = operations.OperationDatasetHistogram(
             inexpr, outbinsds, outvalsds, binparams=binparams,
             binmanual=binmanual, method=method,
             cumulative=cumulative, errors=errors)
@@ -288,7 +306,7 @@ class CommandInterface(qt4.QObject):
 
     def Remove(self, name):
         """Remove a widget from the dataset."""
-        w = self.document.resolve(self.currentwidget, name)
+        w = self.document.resolveWidgetPath(self.currentwidget, name)
         op = operations.OperationWidgetDelete(w)
         self.document.applyOperation(op)
         if self.verbose:
@@ -296,15 +314,22 @@ class CommandInterface(qt4.QObject):
 
     def RemoveCustom(self, name):
         """Removes a custom-defined constant, function or import."""
-        vals = list( self.document.customs )
-        for i, (t, n, v) in enumerate(vals):
-            if n == name:
-                del vals[i]
-                break
+
+        # look for definiton and delete if found
+        for ctype, attr in (
+                ('import', 'def_imports'),
+                ('definition', 'def_definitions'),
+                ('color', 'def_colors'),
+                ('colormap', 'def_colormaps')):
+            vals = list(getattr(self.document.evaluate, attr))
+            for i, (cname, cval) in enumerate(vals):
+                if name == cname:
+                    del vals[i]
+                    op = operations.OperationSetCustom(ctype, vals)
+                    self.document.applyOperation(op)
+                    return
         else:
             raise ValueError('Custom variable not defined')
-        op = operations.OperationSetCustom(vals)
-        self.document.applyOperation(op)
 
     def To(self, where):
         """Change to a widget within the current widget.
@@ -312,8 +337,9 @@ class CommandInterface(qt4.QObject):
         where is a path to the widget relative to the current widget
         """
 
-        self.currentwidget = self.document.resolve(self.currentwidget,
-                                                   where)
+        self.currentwidget = self.document.resolveWidgetPath(
+            self.currentwidget,
+            where)
 
         if self.verbose:
             print(_("Changed to widget '%s'") % self.currentwidget.path)
@@ -321,7 +347,7 @@ class CommandInterface(qt4.QObject):
     def List(self, where='.'):
         """List the contents of a widget, by default the current widget."""
 
-        widget = self.document.resolve(self.currentwidget, where)
+        widget = self.document.resolveWidgetPath(self.currentwidget, where)
         children = widget.childnames
 
         if len(children) == 0:
@@ -334,13 +360,24 @@ class CommandInterface(qt4.QObject):
 
     def Get(self, var):
         """Get the value of a setting."""
-        return self.currentwidget.prefLookup(var).val
+        return self.document.resolveSettingPath(self.currentwidget, var).val
 
     def GetChildren(self, where='.'):
         """Return a list of widgets which are children of the widget of the
         path given."""
-        return list( self.document.resolve(self.currentwidget,
-                                           where).childnames )
+        return list(
+            self.document.resolveWidgetPath(self.currentwidget, where).childnames )
+
+    def GetColormap(self, name, invert=False, nvals=256):
+        """Return an array of [red,green,blue,alpha] values
+        representing the colormap with the name given.
+
+        Each return value is between 0 and 255.
+
+        The number of values to return is given by nvals
+        """
+        cmap = self.document.evaluate.getColormap(name, invert)
+        return utils.getColormapArray(cmap, nvals)
 
     def GetDatasets(self):
         """Return a list of names of datasets."""
@@ -353,9 +390,9 @@ class CommandInterface(qt4.QObject):
         If it is not a reference return None.
         """
 
-        pref = self.currentwidget.prefLookup(setn)
-        if pref.isReference():
-            real = pref.getReference().resolve(pref)
+        setn = self.document.resolveSettingPath(self.currentwidget, setn)
+        if setn.isReference():
+            real = setn.getReference().resolve(setn)
             return real.path
         else:
             return None
@@ -369,33 +406,31 @@ class CommandInterface(qt4.QObject):
         """
         self.document.save(filename, mode)
 
-    def Set(self, var, val):
+    def Set(self, setting_path, val):
         """Set the value of a setting."""
-        pref = self.currentwidget.prefLookup(var)
-
-        op = operations.OperationSettingSet(pref, val)
+        setn = self.document.resolveSettingPath(self.currentwidget, setting_path)
+        op = operations.OperationSettingSet(setn, val)
         self.document.applyOperation(op)
-        
-        if self.verbose:
-            print( _("Set setting '%s' to %s") % (var, repr(pref.get())) )
 
-    def SetToReference(self, var, val):
+        if self.verbose:
+            print( _("Set setting '%s' to %s") % (setting_path, repr(setn.get())) )
+
+    def SetToReference(self, setting_path, val):
         """Set setting to a reference value."""
-
-        pref = self.currentwidget.prefLookup(var)
-        op = operations.OperationSettingSet(pref, setting.Reference(val))
+        setn = self.document.resolveSettingPath(self.currentwidget, setting_path)
+        op = operations.OperationSettingSet(setn, setting.Reference(val))
         self.document.applyOperation(op)
-        
+
         if self.verbose:
-            print( _( "Set setting '%s' to %s") % (var, repr(pref.get())) )
+            print( _( "Set setting '%s' to %s") % (setting_path, repr(setn.get())) )
 
     def SetData(self, name, val, symerr=None, negerr=None, poserr=None):
-        """Set dataset with name with values (and optionally errors)."""
+        """Create/set dataset name with values (and optionally errors)."""
 
         data = datasets.Dataset(val, symerr, negerr, poserr)
         op = operations.OperationDatasetSet(name, data)
         self.document.applyOperation(op)
- 
+
         if self.verbose:
             print(
                 _("Set dataset '%s':\n"
@@ -432,7 +467,7 @@ class CommandInterface(qt4.QObject):
         are used, and so on.
         To access a specific part of the dataset y, the suffixes _data, _serr, _perr,
         and _nerr can be appended.
-        
+
         If linked is True then the expressions are reevaluated if the document
         is modified
 
@@ -446,7 +481,7 @@ class CommandInterface(qt4.QObject):
                                                          parametric=parametric)
 
         data = self.document.applyOperation(op)
-        
+
         if self.verbose:
             print(
                 _("Set dataset '%s' based on expression:\n"
@@ -461,6 +496,20 @@ class CommandInterface(qt4.QObject):
                 print(_(" Where t goes form %g:%g in %i steps") % parametric)
             print(_(" linked to expression = %s") % repr(linked))
 
+    def SetDataND(self, name, val):
+        """Set n-dimensional dataset name with values."""
+
+        data = datasets.DatasetND(val)
+        op = operations.OperationDatasetSet(name, data)
+        self.document.applyOperation(op)
+
+        if self.verbose:
+            print(
+                _("Set dataset (nD) '%s':\n"
+                  " Values = %s\n") % (
+                      name, str(data.data))
+            )
+
     def SetDataRange(self, name, numsteps, val, symerr=None, negerr=None,
                      poserr=None, linked=False):
         """Create dataset based on ranges of values, e.g. 1 to 10 in 10 steps
@@ -475,7 +524,7 @@ class CommandInterface(qt4.QObject):
         op = operations.OperationDatasetCreateRange(name, numsteps, parts,
                                                     linked)
         self.document.applyOperation(op)
-        
+
         if self.verbose:
             print(
                 _("Set dataset '%s' based on range:\n"
@@ -483,9 +532,9 @@ class CommandInterface(qt4.QObject):
                   " Range of data = %s\n"
                   " Range of symmetric error = %s\n"
                   " Range of positive error = %s\n"
-                  " Range of negative error = %s" % (
+                  " Range of negative error = %s") % (
                       name, numsteps, repr(val),
-                      repr(symerr), repr(poserr), repr(negerr)) )
+                      repr(symerr), repr(poserr), repr(negerr))
               )
 
     def SetData2DExpression(self, name, expr, linked=False):
@@ -623,6 +672,7 @@ class CommandInterface(qt4.QObject):
             (data, serr, nerr, perr)
         For a 2D dataset, returns
             (data, xrange, yrange)
+        For an nD dataset returns data array
         For a text dataset, return a list of text
         For a date dataset, return a list of python datetime objects
 
@@ -636,6 +686,8 @@ class CommandInterface(qt4.QObject):
             return [utils.floatToDateTime(x) for x in d.data]
         elif d.dimensions == 2:
             return (d.data.copy(), d.xrange, d.yrange)
+        elif d.dimensions == -1:
+            return d.data.copy()
         else:
             data = serr = nerr = perr = None
             if d.data is not None:
@@ -685,7 +737,7 @@ class CommandInterface(qt4.QObject):
     def Action(self, action, widget='.'):
         """Performs action on current widget."""
 
-        w = self.document.resolve(self.currentwidget, widget)
+        w = self.document.resolveWidgetPath(self.currentwidget, widget)
 
         # run action
         w.getAction(action).function()
@@ -693,14 +745,14 @@ class CommandInterface(qt4.QObject):
     def Print(self):
         """Print document."""
         export.printDialog(None, self.document)
-            
-    def Export(self, filename, color=True, page=0, dpi=100,
+
+    def Export(self, filename, color=True, page=[0], dpi=100,
                antialias=True, quality=85, backcolor='#ffffff00',
                pdfdpi=150, svgtextastext=False):
         """Export plot to filename.
 
         color is True or False if color is requested in output file
-        page is the pagenumber to export
+        page is a list of page numbers to export
         dpi is the number of dots per inch for bitmap output files
         antialias antialiases output if True
         quality is a quality parameter for jpeg output
@@ -709,11 +761,18 @@ class CommandInterface(qt4.QObject):
         pdfdpi is the dpi to use when exporting eps or pdf files
         svgtextastext: write text in SVG as text, rather than curves
         """
-        
-        e = export.Export(self.document, filename, page, color=color,
-                          bitmapdpi=dpi, antialias=antialias,
-                          quality=quality, backcolor=backcolor,
-                          pdfdpi=pdfdpi, svgtextastext=svgtextastext)
+
+        # compatibility where page was a single number
+        try:
+            pages = [p for p in page]
+        except TypeError:
+            pages = [page]
+
+        e = export.Export(
+            self.document, filename, pages, color=color,
+            bitmapdpi=dpi, antialias=antialias,
+            quality=quality, backcolor=backcolor,
+            pdfdpi=pdfdpi, svgtextastext=svgtextastext)
         e.export()
 
     def Rename(self, widget, newname):
@@ -722,7 +781,7 @@ class CommandInterface(qt4.QObject):
         eg Rename('graph1/xy1', 'scatter')
         This function does not move widgets."""
 
-        w = self.document.resolve(self.currentwidget, widget)
+        w = self.document.resolveWidgetPath(self.currentwidget, widget)
         op = operations.OperationWidgetRename(w, newname)
         self.document.applyOperation(op)
 
@@ -733,11 +792,11 @@ class CommandInterface(qt4.QObject):
         Returns type of node given.
         Return values are: 'widget', 'settings' or 'setting'
         """
-        item = self.document.resolveItem(self.currentwidget, path)
+        item = self.document.resolvePath(self.currentwidget, path)
 
-        if hasattr(item, 'isWidget') and item.isWidget():
+        if item.iswidget:
             return 'widget'
-        elif isinstance(item, setting.Settings):
+        elif item.issettings:
             return 'settinggroup'
         else:
             return 'setting'
@@ -748,17 +807,17 @@ class CommandInterface(qt4.QObject):
 
         Returns a list of the names of the children of this node."""
 
-        item = self.document.resolveItem(self.currentwidget, path)
+        item = self.document.resolvePath(self.currentwidget, path)
 
         out = []
-        if hasattr(item, 'isWidget') and item.isWidget():
+        if item.iswidget:
             if types == 'all' or types == 'widget':
                 out += item.childnames
             if types == 'all' or types == 'settinggroup':
                 out += [s.name for s in item.settings.getSettingsList()]
             if types == 'all' or types == 'setting':
                 out += [s.name for s in item.settings.getSettingList()]
-        elif isinstance(item, setting.Settings):
+        elif item.issettings:
             if types == 'all' or types == 'settinggroup':
                 out += [s.name for s in item.getSettingsList()]
             if types == 'all' or types == 'setting':
@@ -770,19 +829,19 @@ class CommandInterface(qt4.QObject):
 
         Raises a ValueError if the path doesn't point to a widget."""
 
-        item = self.document.resolveItem(self.currentwidget, path)
-        if hasattr(item, 'isWidget') and item.isWidget():
+        item = self.document.resolvePath(self.currentwidget, path)
+        if item.iswidget:
             return item.typename
         else:
             raise ValueError("Path '%s' is not a widget" % path)
 
-    def SettingType(self, path):
+    def SettingType(self, setting_path):
         """Get the type of setting (a string) for the path given.
 
         Raise a ValueError if path is not a setting
         """
 
-        setn = self.currentwidget.prefLookup(path)
+        setn = self.document.resolveSettingPath(self.currentwidget, setting_path)
         return setn.typename
 
     def TagDatasets(self, tag, datasets):
@@ -793,3 +852,27 @@ class CommandInterface(qt4.QObject):
         if self.verbose:
             print(_("Applied tag %s to datasets %s") % (
                 tag, ' '.join(datasets)))
+
+    def FilterDatasets(self, filterexpr, dataset_list,
+                       prefix="", suffix="",
+                       invert=False, replaceblanks=False):
+        """Apply filter expression to list of datasets.
+
+        filterexpr: input filter expression
+        dataset_list: list of input dataset names
+        prefix, suffix: output prefix/suffix to add to names (one must be set)
+        invert: invert results of filter expression
+        replaceblanks: replace filtered values with nan/blank in output.
+        """
+        op = operations.OperationDatasetsFilter(
+            filterexpr, dataset_list,
+            prefix=prefix, suffix=suffix,
+            invert=invert, replaceblanks=replaceblanks)
+        self.document.applyOperation(op)
+
+        if self.verbose:
+            print(
+                _('Filtered datasets %s using expression %s. '
+                  'Output prefix=%s, suffix=%s') % (
+                      dataset_list, filterexpr, prefix, suffix)
+            )
