@@ -20,6 +20,7 @@
 
 from __future__ import division, print_function
 import sys
+import datetime
 
 import numpy as N
 from .. import qtall as qt4
@@ -59,6 +60,9 @@ defaultValues = {
     'color_cntrlline': (True, 'blue'),
     'color_cntrlcorner': (True, 'black'),
 
+    # document theme
+    'colortheme_default': 'default-latest',
+
     # further ui options
     'toolbar_size': 24,
 
@@ -68,12 +72,41 @@ defaultValues = {
     # use cwd as starting directory
     'dirname_usecwd': False,
 
+    # use document directory for export
+    'dirname_export_location': 'doc',
+
+    # export templates
+    'export_template_single': '%DOCNAME%',
+    'export_template_multi': '%DOCNAME%_%PAGE00%',
+
     # ask tutorial before?
     'ask_tutorial': False,
 
     # log picked points to clipboard or to console
     'picker_to_clipboard': False,
-    'picker_to_console': True
+    'picker_to_console': True,
+
+    # add these directories to the python path (colon-separated)
+    'external_pythonpath': '',
+
+    # location of ghostscript (or empty to search)
+    'external_ghostscript': '',
+
+    # translation file to load
+    'translation_file': '',
+
+    # user has disabled version update checks
+    # (packagers: please don't disable here, see disableVersionChecks in
+    #  veusz/utils/version.py)
+    'vercheck_disabled': False,
+    'vercheck_asked_user': False,
+    'vercheck_last_done': (2000,1,1),
+    'vercheck_latest': '',
+
+    # whether to send feedback about usage
+    # (packagers, please don't disable here but in veusz/setting/feedback.py)
+    'feedback_disabled': False,
+    'feedback_asked_user': False,
     }
 
 class _SettingDB(object):
@@ -149,6 +182,11 @@ class _SettingDB(object):
             if key not in self.database:
                 self.database[key] = defaultValues[key]
 
+        # keep install date for reminders, etc
+        if 'install_date' not in self.database:
+            today = datetime.date.today()
+            self.database['install_date'] = (today.year, today.month, today.day)
+
     def writeSettings(self):
         """Write the settings using QSettings.
 
@@ -194,8 +232,10 @@ class _SettingDB(object):
 settingdb = _SettingDB()
 
 # a normal dict for non-persistent settings
-# (e.g. disable safe mode)
-transient_settings = {}
+transient_settings = {
+    # disable safety checks on evaluated code
+    'unsafe_mode': False,
+}
 
 def updateUILocale():
     """Update locale to one given in preferences."""
@@ -209,7 +249,7 @@ def updateUILocale():
 
     qt4.QLocale.setDefault(uilocale)
 
-def ui_floattostring(f):
+def ui_floattostring(f, maxdp=14):
     """Convert float to string with more precision."""
     if not N.isfinite(f):
         if N.isnan(f):
@@ -218,12 +258,12 @@ def ui_floattostring(f):
             return '-inf'
         return 'inf'
     elif 1e-4 <= abs(f) <= 1e5 or f == 0:
-        s = '%.14g' % f
+        s = ('%.'+str(maxdp)+'g') % f
         # strip excess zeros to right
         if s.find('.') >= 0:
             s = s.rstrip('0').rstrip('.')
     else:
-        s = '%.14e' % f
+        s = ('%.'+str(maxdp)+'e') % f
         # split into mantissa/exponent and strip extra zeros, etc
         mant, expon = s.split('e')
         mant = mant.rstrip('0').rstrip('.')
