@@ -144,14 +144,6 @@ class Image(plotters.GenericPlotter):
     allowusercreation=True
     description=_('Plot a 2d dataset as an image')
 
-    def __init__(self, parent, name=None):
-        """Initialise plotter with axes."""
-
-        plotters.GenericPlotter.__init__(self, parent, name=name)
-
-        if type(self) == Image:
-            self.readDefaults()
-
     @classmethod
     def addSettings(klass, s):
         """Construct list of settings."""
@@ -231,11 +223,17 @@ class Image(plotters.GenericPlotter):
 
         s = self.settings
         minval = s.min
-        if minval == 'Auto' and data is not None:
-            minval = N.nanmin(data.data)
+        if minval == 'Auto':
+            if data is not None:
+                minval = N.nanmin(data.data)
+            else:
+                minval = 0.
         maxval = s.max
-        if maxval == 'Auto' and data is not None:
-            maxval = N.nanmax(data.data)
+        if maxval == 'Auto':
+            if data is not None:
+                maxval = N.nanmax(data.data)
+            else:
+                maxval = minval + 1
 
         # this is used currently by colorbar objects
         return (minval, maxval)
@@ -294,8 +292,13 @@ class Image(plotters.GenericPlotter):
         pltrangex = axes[0].dataToPlotterCoords(posn, N.array(rangex))
         pltrangey = axes[1].dataToPlotterCoords(posn, N.array(rangey))
 
+        # abort if coordinate range is too small
+        if(abs(pltrangex[0]-pltrangex[1])<1e-2 or
+           abs(pltrangey[0]-pltrangey[1])<1e-2):
+            return
+
         # make QImage from data
-        cmap = d.getColormap(s.colorMap, s.colorInvert)
+        cmap = d.evaluate.getColormap(s.colorMap, s.colorInvert)
         datavaluerange = self.getDataValueRange(data)
         image = utils.applyColorMap(
             cmap, s.colorScaling, data.data,
